/* File bsb_list.c */

/*--------------------------------------------------------------------
Lists the stimuli on the screen.  Because this program can be
modified for heteroassociation, both the f (input) and g(output)
sets can be shown.
--------------------------------------------------------------------*/

#include <stdio.h>      /* fgets printf */

/* Number_of_characters is related to Dimensionality of vectors */
#define Number_of_characters (int)(25)
#define Dimensionality       (int)(8*Number_of_characters)
/* Maximum_set_size is the size of each training set */
#define Maximum_set_size     (int)(100)
/* String_length refers to strings in the struct Stimulus.Name */
#define String_length   (int)(60)       /* Must be > Number_of_characters */
#define VT100            /* remove if you do not have VT100 compatible term */

typedef float Vector[Dimensionality];
typedef char String[String_length+1];
typedef struct {
    String Name;
    Vector Val;
} Stimulus;

/* Defined in bsb.c */
int FG_mode;
Stimulus F_set[Maximum_set_size+1];
Stimulus G_set[Maximum_set_size+1];
Stimulus T_set[Maximum_set_size+1];

/* Defined in bsb_list.c */
static int Backward,Forward;	/* Direction to move within list */
static int Quit;		/* exit list routine */
static void Write_message(int Biggest);
static void Write_pair(char One, char Two, int i);
static void List_screen(char One, char Two);
#if 0
static void Type_list(char One, char Two);
#endif
void List_test_set(void);

/* Defined in bsb_num.c */
int Number_present_in_set(char Which_set);

/* Defined in bsb_vt100.c */
void Save_cursor(void);
void Bottom_line(void);
void Restore_cursor(void);
void Clear_block(void);
void Scroll_block(void);
 
/*--------------------------------------------------------------------
--------------------------------------------------------------------*/
static void Write_message(int Biggest) {
    char InputLine[80];	/* keyboard input */
    char Input_char;	/* keyboard input */

    Save_cursor();
    Bottom_line();
    Backward=Forward=Quit=0;	/* Set all flags to FALSE */
    printf("There are %3d stimuli to write. B)ack. F)orward. > ",Biggest);
    fgets(InputLine,sizeof(InputLine),stdin);
    Input_char=InputLine[0];

    if(Input_char=='B' || Input_char=='b')
        Backward=1;
    else if(Input_char=='F' || Input_char=='f')
        Forward=1;
    else
        Quit=1;
    Restore_cursor();
}   /* Write_message */


/*--------------------------------------------------------------------
--------------------------------------------------------------------*/
static void Write_pair(char One, char Two, int i) {
    int j;	/* loop index */
    char *Name_one;	/* pointer to first stim string */
    char *Name_two;	/* pointer to second stim string */

    if(One=='F' || One=='f')      Name_one=F_set[i].Name;
    else if(One=='G' || One=='g') Name_one=G_set[i].Name;
    else if(One=='T' || One=='t') Name_one=T_set[i].Name;

    if(Two=='F' || Two=='f')      Name_two=F_set[i].Name;
    else if(Two=='G' || Two=='g') Name_two=G_set[i].Name;
    else if(Two=='T' || Two=='t') Name_two=T_set[i].Name;

    printf("   %c[%2d]. ",One,i);
    for(j=0; j<Number_of_characters; j++) printf("%c",Name_one[j]);
    printf("   %c[%2d]. ",Two,i);
    for(j=0; j<Number_of_characters; j++) printf("%c",Name_two[j]);
    printf("\n");
}   /* Write_pair */


/*--------------------------------------------------------------------
--------------------------------------------------------------------*/
static void List_screen(char One, char Two) {
    int i;	/* loop index */
    int First,Last;	/* range of lines to list */
    int Largest;	/* larger of the two list lengths */
    int Last_pair;	/* like Last... */

    /* Initialize local variables */
    First=1;
    Largest=Number_present_in_set(One);
    if(Number_present_in_set(Two)>Largest)
        Largest=Number_present_in_set(Two);
    Last_pair=Largest;
    Last=(Last_pair>16)? 16: Last_pair;

    /* List */
    for(i=First; i<=Last; i++)
        Write_pair(One,Two,i);

    /* Get input */
    if(Last_pair>16) {
        do {
            Write_message(Largest);
            if(Forward && !Quit) {
                First=Last+1;
                Last=Last+8;
                if(Last>Last_pair) Last=Last_pair;
                for(i=First; i<=Last; i++) Write_pair(One,Two,i);
                First=Last-15;
            } else if(Backward && !Quit) {
                First=First-8;
                if(First<1) First=1;
                Last=First+15;
                if(Last>Last_pair) {
                    Last=Last_pair;
                    First=Last_pair-15;
                }
                Clear_block();
                printf("\n");
                for(i=First; i<=Last; i++) Write_pair(One,Two,i);
            }
        } while(!Quit);
    }
}   /* List_screen */


#if !defined VT100
/*--------------------------------------------------------------------
To be used if do not have VT-100.
--------------------------------------------------------------------*/
static void Type_list(char One, char Two) {
    int i;	/* loop index */
    int Largest;	/* larger of the two list lengths */

    Largest=Number_present_in_set(One);
    if(Number_present_in_set(Two)>Largest)
        Largest=Number_present_in_set(Two);

    /* List */
    for(i=1; i<=Largest; i++)
        Write_pair(One,Two,i);
}   /* Type_list */
#endif

/*--------------------------------------------------------------------
--------------------------------------------------------------------*/
void List_test_set(void) {
    Scroll_block();
    Clear_block();
    printf("\n");
    if(FG_mode) List_screen('F','G');
    else        List_screen('T','F');
#if !defined VT100
    if(FG_mode) Type_list('F','G');
    else        Type_list('T','F');
#endif
}   /* List_test_set */

