/* File bsb_num.c */

/*--------------------------------------------------------------------
Some useful procedures for initialization and to tell how many
non-blank stimului are present in the various data files.
--------------------------------------------------------------------*/

/* Number_of_characters is related to Dimensionality of vectors */
#define Number_of_characters (int)(25)
#define Dimensionality       (int)(8*Number_of_characters)
/* Maximum_set_size is the size of each training set */
#define Maximum_set_size     (int)(100)
/* String_length refers to strings in the struct Stimulus.Name */
#define String_length   (int)(60)

typedef float Vector[Dimensionality];
typedef char String[String_length+1];
typedef struct {
    String Name;
    Vector Val;
} Stimulus;

/* Defined in bsb.c */
/* Element [0] does not appear to be used anywhere */
Stimulus F_set[Maximum_set_size+1];
Stimulus G_set[Maximum_set_size+1];
Stimulus T_set[Maximum_set_size+1];

/* Defined in bsb_num.c */
void Zero_vector(Vector V);
void Blank_string(String S);
void Blank_f_set(void);
void Blank_g_set(void);
void Blank_t_set(void);
static int All_blanks(String S);
int Number_present_in_set(char Which_set);

/*------------------------------------------------------------------------
Sets the array of <Dimensionality> floats to zero.
------------------------------------------------------------------------*/
void Zero_vector(Vector V) {
    int i;      /* loop index */

    for(i=0; i<Dimensionality; i++)
        V[i]=0;
}   /* Zero_vector */

/*------------------------------------------------------------------------
Sets the array of <String_length> chars to ' '.
------------------------------------------------------------------------*/
void Blank_string(String S) {
    int i;      /* loop index */

    for(i=0; i<String_length; i++)
        S[i]=' ';
    S[String_length]='\0';
}   /* Blank_string */

/*--------------------------------------------------------------------
Write a blank string, 60 chars long, to elements 1..<Maximum_set_size>
of F_set[].Name.  Element 0 is ignored.
--------------------------------------------------------------------*/
void Blank_f_set(void) {
    int i;      /* loop index */

    for(i=1; i<=Maximum_set_size; i++)
        Blank_string(F_set[i].Name);
}   /* Blank_f_set */

/*--------------------------------------------------------------------
Write a blank string, 60 chars long, to elements 1..<Maximum_set_size>
of G_set[].Name.  Element 0 is ignored.
--------------------------------------------------------------------*/
void Blank_g_set(void) {
    int i;      /* loop index */

    for(i=1; i<=Maximum_set_size; i++)
        Blank_string(G_set[i].Name);
}   /* Blank_g_set */

/*--------------------------------------------------------------------
Write a blank string, 60 chars long, to elements 1..<Maximum_set_size>
of T_set[].Name.  Element 0 is ignored.
--------------------------------------------------------------------*/
void Blank_t_set(void) {
    int i;      /* loop index */

    for(i=1; i<=Maximum_set_size; i++)
        Blank_string(T_set[i].Name);
}   /* Blank_t_set */


/*--------------------------------------------------------------------
Examines the specified string, and returns TRUE if all characters
are spaces.  Otherwise it returns FALSE.
--------------------------------------------------------------------*/
static int All_blanks(String S) {
    int i;	/* loop index */

    for(i=0; i<Number_of_characters; i++) 
        if(S[i]!=' ') return 0;		/* FALSE */
    return 1;				/* TRUE */
}   /* All_blanks */


/*--------------------------------------------------------------------
The charater <Which_set> equals 'f', 'g' or 't', and selects one of
the arrays F_set[], G_set[] or T_set[].  The values X_set[i].Name
i=1..Maximum_set_size are initialized to blank strings, and then
fill from the begining with strings read from files.  This routine
determines the number of strings present in the selected array
by looking for a non-blank string, starting from the end.  The 
value returned can have a value from 0 to Maximum_set_size.
--------------------------------------------------------------------*/
int Number_present_in_set(char Which_set) {
    int i;	/* loop index */

    if(Which_set=='f' || Which_set=='F') {
        for(i=Maximum_set_size; i>0; i--) 
            if(!All_blanks(F_set[i].Name)) return i;
    } else if(Which_set=='g' || Which_set=='G') {
        for(i=Maximum_set_size; i>0; i--) 
            if(!All_blanks(G_set[i].Name)) return i;
    } else if(Which_set=='t' || Which_set=='T') {
        for(i=Maximum_set_size; i>0; i--)
            if(!All_blanks(T_set[i].Name)) return i;
    } else {
        /* Illegal value of Which_set */
    }
    return 0;
}   /* Number_present_in_set */

