/* File bsb_vt100.c */

/*--------------------------------------------------------------------------
This is a series of screen control commands for a VT100 terminal or 
equivalent.  It should work on most terminals or common terminal 
emulators.
--------------------------------------------------------------------------*/
#if defined MSDOS
#include <graphics.h>   /* _clearscreen _settextposition */
#else
#include <stdio.h>	/* printf */
#endif

/* Defined in bsb_vt100.c */
void Clear_screen(void);
void Clear_box(void);
void Scroll_box(void);
void Clear_block(void);
void Erase_line(void);
void Top_of_block(void);
void Bottom_of_block(void);
void Scroll_block(void);
void Save_cursor(void);
void Restore_cursor(void);
void Erase_to_end(void);
void Bottom_line(void);
void Restore_scrolling(void);
void GoToXY(int x, int y);

/*--------------------------------------------------------------------------
Screen Control Procedure: Clear screen and move cursor to 1,1
--------------------------------------------------------------------------*/
void Clear_screen(void) {
#if defined MSDOS
    _clearscreen(_GCLEARSCREEN);
#else
    printf("%c[2J",0x1b);
    printf("%c[%d;%dH",0x1b,1,1);
#endif
}   /* ClrScr */

/*--------------------------------------------------------------------------
Screen Control Procedure: Clear first five lines of screen and move 
cursor to 1,1
--------------------------------------------------------------------------*/
void Clear_box(void) {
#if defined MSDOS
#else
    printf("%c[%d;%df",0x1b,5,80);	/* move cursor to 5,80 */
    printf("%c[1J",0x1b);		/* erase from 1,1 to cursor */
    printf("%c[f",0x1b);		/* move curor to 1,1 */
#endif
}   /* Clear_box */

/*--------------------------------------------------------------------------
Screen Control Procedure: Define first five lines of screen as a scrolling
region, and move cursor to 1,1 
--------------------------------------------------------------------------*/
void Scroll_box(void) {
#if defined MSDOS
#else
    printf("%c[%d;%dr",0x1b,1,5);	/* define scrolling region */
    printf("%c[f",0x1b);		/* move curor to 1,1 */
#endif
}   /* Scroll_box */

/*--------------------------------------------------------------------------
Screen Control Procedure: Clear lines 6..24 of screen and move 
cursor to 6,1
--------------------------------------------------------------------------*/
void Clear_block(void) {
#if defined MSDOS
#else
    printf("%c[%d;%df",0x1b,6,1);	/* move cursor to 6,1 */
    printf("%c[0J",0x1b);		/* erase from cursor to 24,80 */
#endif
}   /* Clear_block */

/*--------------------------------------------------------------------------
Screen Control Procedure: Erase from cursor to end of line
--------------------------------------------------------------------------*/
void Erase_line(void) {
#if defined MSDOS
#else
    printf("%c[K",0x1b);		/* erase from cursor to n,80 */
#endif
}   /* Erase_line */

/*--------------------------------------------------------------------------
Screen Control Procedure: Moves cursor to beginning of line 6 
--------------------------------------------------------------------------*/
void Top_of_block(void) {
#if defined MSDOS
#else
    printf("%c[%d;%df",0x1b,6,1);	/* move to 6,1 */
#endif
}   /* Top_of_block */

/*--------------------------------------------------------------------------
Screen Control Procedure: Moves cursor to beginning of line 23 
--------------------------------------------------------------------------*/
void Bottom_of_block(void) {
#if defined MSDOS
#else
    printf("%c[%d;%df",0x1b,23,1);	/* move to 23,1 */
#endif
}   /* Bottom_of_block */


/*--------------------------------------------------------------------------
Screen Control Procedure: Define lines 6 - 23 of screen as a scrolling
region, and move cursor to 6,1 
--------------------------------------------------------------------------*/
void Scroll_block(void) {
#if defined MSDOS
#else
    printf("%c[%d;%dr",0x1b,6,23);	/* define scrolling region */
    printf("%c[%d;%df",0x1b,6,1);	/* move curor to 6,1 */
#endif
}   /* Scroll_block */

/*--------------------------------------------------------------------------
Screen Control Procedure: Save the cursor position
--------------------------------------------------------------------------*/
void Save_cursor(void) {
#if defined MSDOS
#else
    printf("%c%d",0x1b,7);	/* save cursor position */
#endif
}   /* Save_cursor */

/*--------------------------------------------------------------------------
Screen Control Procedure: Restore the cursor position
--------------------------------------------------------------------------*/
void Restore_cursor(void) {
#if defined MSDOS
#else
    printf("%c%d",0x1b,8);		/* restore cursor position */
#endif
}   /* Restore_cursor */

/*--------------------------------------------------------------------------
Screen Control Procedure: Erase from cursor to end of line
--------------------------------------------------------------------------*/
void Erase_to_end(void) {
#if defined MSDOS
#else
    printf("%c[K",0x1b);		/* erase from cursor to n,80 */
#endif
}   /* Erase_to_end */

/*--------------------------------------------------------------------------
Screen Control Procedure: Move to and erase bottom line
--------------------------------------------------------------------------*/
void Bottom_line(void) {
#if defined MSDOS
#else
    printf("%c[%d;%df",0x1b,24,1);	/* move to 24,1 */
    printf("%c[0K",0x1b);		/* erase from cursor to 24,80 */
#endif
}   /* Bottom_line */

/*--------------------------------------------------------------------------
Screen Control Procedure: Define lines 1 - 24 of screen as a scrolling
region 
--------------------------------------------------------------------------*/
void Restore_scrolling(void) {
#if defined MSDOS
#else
    printf("%c[%d;%dr",0x1b,1,24);	/* define scrolling region */
#endif
}   /* Restore_scrolling */

/*--------------------------------------------------------------------------
Screen Control Procedure: Position cursor

Arguments
    x    column as integer 1..80
    y    row as integer 1..24
--------------------------------------------------------------------------*/
void GoToXY(int x, int y) {
    int x1,y1;          /* coordinates restricted to allowed range */

    x1=(x<1)? 1: (x>80)? 80: x;         /* 1<=x1<=80 */
    y1=(y<1)? 1: (y>24)? 24: y;         /* 1<=y1<=24 */
#if defined MSDOS
    _settextposition(y,x);
#else
    printf("%c[%d;%dH",0x1b,y1,x1);
#endif
}   /* GoToXY */

