/* File bsb.c, Program bsb */

/*-----------------------------------------------------------------------
Copyright 1994 James Anderson, Brown University.                       
This research was primarily supported by National Science
Foundation Grants BNS-82-14728, BNS-85-18675 and BNS-90-23283
to: James Anderson
    Department of Cognitive and Linguistic Sciences
    Box 1978
    Brown University, Providence, RI 02912.

    James_Anderson@brown.edu

Please acknowledge these grants if you make use of these programs in
published material. 

A version of this program is used for the computer simulations in
An Introduction to Neural Networks, by James A. Anderson, MIT Press.
-----------------------------------------------------------------------*/

#if defined MSDOS
#include <io.h>         /* access */
#else
#include <unistd.h>     /* access */
#endif
#include <ctype.h>	/* toupper */
#include <math.h>       /* fabs */
#include <stdio.h>      /* fgets getchar printf scanf */
#include <string.h>     /* strncpy */

/* Number_of_characters is related to Dimensionality of vectors */
#define Number_of_characters (int)(25)
#define Dimensionality       (int)(8*Number_of_characters)
/* Maximum_set_size is the size of each training set */
#define Maximum_set_size     (int)(100)
/* String_length refers to strings in the struct Stimulus.Name */
#define String_length  (int)(60)	/* Must be > Number_of_characters */
#define FALSE 0
#define TRUE  1

typedef float Vector[Dimensionality];
typedef char String[String_length+1];
typedef struct {
    String Name;
    Vector Val;
} Stimulus;
typedef struct {
    float Strength;
    int From;
} Synapse;
typedef struct {
    float Activity;
    Synapse Synapses[Dimensionality];
    float Neuron_output;
    float Learning;
    float Neuron_upper_limit;
    float Neuron_lower_limit;
} Neuron;

/* Defined in bsb.c */
/* Element [0] does not appear to be used anywhere */
Stimulus F_set[Maximum_set_size+1];
Stimulus G_set[Maximum_set_size+1];
Stimulus T_set[Maximum_set_size+1];
Neuron Neurons[Dimensionality];
int Neuron_file_present;
int Number_in_test_file;
float Threshold;
int FG_mode;
int Exit;
char FFileName[FILENAME_MAX];
char GFileName[FILENAME_MAX];
char TFileName[FILENAME_MAX];
char NFileName[FILENAME_MAX];
static void Set_threshold(void);
static void File_initialize(void);
static void Help_info(void);
static void Do_option(char Option);
static void Get_command_string(void);
static void Initialize(void);
void main(int argc, char **argv);

/* Defined in bsb_list.c */
void List_test_set(void);

/* Defined in bsb_make.c */
void Init_template(void);
void Make_stimulus(void);

/* Defined in bsb_num.c */
void Blank_f_set(void);
void Blank_g_set(void);
void Blank_t_set(void);
int Number_present_in_set(char Which_set);

/* Defined in bsb_proc.c */
void BSBProcedure(void);

/* Defined in bsb_readf.c */
void Read_file(char Read_which_file);

/* Defined in bsb_readnf.c */
int Avg_number_of_synapses;
void Read_neuron_file(void);

/* Defined in bsb_vt100.c */
void Clear_screen(void);
void Clear_box(void);
void Scroll_box(void);
void Clear_block(void);
void Erase_line(void);
void Top_of_block(void);
void Save_cursor(void);
void Restore_cursor(void);
void Bottom_line(void);
void Restore_scrolling(void);

/* Defined in bsb_writef.c */
void Write_file(char Which_file);

/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Set_threshold(void) {
    printf("  Old threshold : %.3f   Set threshold : ",Threshold);
    scanf("%f",&Threshold);
    getchar();
}   /* Set_threshold */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void File_initialize(void) {
    Clear_block();
    Top_of_block();
    printf("Initializing files.\n");
    printf("Reading neuron and stimulus files from disk.\n");

    printf("Reading F file \'%s\'\n",FFileName);
    Save_cursor(); Read_file('F'); Restore_cursor();

    printf("Reading G file \'%s\'\n",GFileName);
    Save_cursor(); Read_file('G'); Restore_cursor();

    printf("Reading T file \'%s\'\n",TFileName);
    Save_cursor(); Read_file('T'); Restore_cursor();

    printf("Reading autoassociative neuron file \'%s\'\n",NFileName);
    Save_cursor(); Read_neuron_file(); Restore_cursor();
}   /* File_initialize */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Help_info(void) {
    Clear_box();
    Scroll_box();

    printf("BSB Neural Net Program. 10-94   B)SB.  C)hange. E)xit. H)elp.\n");
    printf("L)ist. M)ode. N)eurons. R)ead. S)ave. T)hreshold.  W)rite.  \n");
    printf("Threshold: %.3f  ",Threshold);
    printf("F File: %4d  ",Number_present_in_set('F'));
    printf("G File: %4d  ",Number_present_in_set('G'));
    printf("T File: %4d\n",Number_present_in_set('T'));
    if(Neuron_file_present)
        printf("Synapses : %4d",Avg_number_of_synapses);
    else
        printf("NO NEURON FILE.");
    if(FG_mode)
        printf("  Display FG.\n");
    else
        printf("  Display TF.\n");
}   /* Help_info */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Do_option(char Option) {
    char Input_char;
    char Which_file;

    if(Option=='B' || Option=='b') {
        if(Neuron_file_present) BSBProcedure();
        else                    printf("No neuron file present.");
    } else if(Option=='C' || Option=='c') {
        Make_stimulus();
    } else if(Option=='E' || Option=='e') {
        Exit=TRUE;
    } else if(Option=='F' || Option=='f') {
        do {
            Bottom_line();
            Erase_line();
            printf("Reinitialize files? Y/N : ");
            Input_char=toupper(getchar());
        } while(Input_char!='Y' && Input_char!='N' && Input_char!=0x1b);
        if(Input_char=='Y' || Input_char=='y') File_initialize();
        else                                   printf("No reinitialization.");
    } else if(Option=='H' || Option=='h') {
        Help_info();
    } else if(Option=='L' || Option=='l') {
        List_test_set();
    } else if(Option=='M' || Option=='m') {
        FG_mode=!FG_mode;
    } else if(Option=='N' || Option=='n') {
        Bottom_line();
        printf("Read neuron file ");
        Read_neuron_file();
    } else if(Option=='R' || Option=='r') {
        do {
            Bottom_line();
            printf("Read which file: F, G, T or A)ll? : ");
            Which_file=toupper(getchar());
        } while(Which_file!='F' && Which_file!='G' &&
                Which_file!='T' && Which_file!='A' &&
                Which_file!=0x1b);
        if(Which_file!=0x1b) 
             Read_file(Which_file);
    } else if(Option=='S' || Option=='s') {
        do {
            Bottom_line();
            printf("Store which file: F, G or T? : ");
            Which_file=toupper(getchar());
        } while(Which_file!='F' && Which_file!='G' &&
                Which_file!='T' && Which_file!=0x1b);
        if(Which_file!=0x1b) 
             Write_file(Which_file);
    } else if(Option=='T' || Option=='t') {
        Set_threshold();
    }
}   /* Do_option */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Get_command_string(void) {
    char Command_string[132];
    char *p;		/* character from Command_string */

    /* Show prompt and get string */
    Help_info();
    Bottom_line();
    printf("C>");
    fgets(Command_string,sizeof(Command_string)-1,stdin);

    /* If <Esc> pressed, discard entire line */
    for(p=Command_string; *p; p++) if(*p==0x1b) return;

    /* Process all commands.  Illegal characters will be ignored */
    for(p=Command_string; *p; p++) Do_option(*p);
}   /* Get_command_string */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Initialize(void) {
    Clear_screen();
    Blank_f_set();
    Blank_g_set();
    Blank_t_set();
    Init_template();
    Threshold=0.5;
    Neuron_file_present=FALSE;
    FG_mode=FALSE;
    Exit=FALSE;
}   /* Initialize */


/*-----------------------------------------------------------------------
<argc> can equal anything from 1 (no file names supplied) to 5 (all four
file names supplied).
-----------------------------------------------------------------------*/
void main(int argc, char *argv[]) {
    if(argc>1) strncpy(FFileName,argv[1],sizeof(FFileName));
    else       strncpy(FFileName,"Ffile",sizeof(FFileName));
    if(argc>2) strncpy(GFileName,argv[2],sizeof(GFileName));
    else       strncpy(GFileName,"Gfile",sizeof(GFileName));
    if(argc>3) strncpy(TFileName,argv[3],sizeof(TFileName));
    else       strncpy(TFileName,"Tfile",sizeof(TFileName));
    if(argc>4) strncpy(NFileName,argv[4],sizeof(NFileName));
    else       strncpy(NFileName,"Nfile",sizeof(NFileName));

    Initialize();
    File_initialize();
    do {
        Get_command_string();
    } while(!Exit);
    Restore_scrolling();
    Clear_screen();
}   /* main */
