/* File bsb_readf.c */

/*-----------------------------------------------------------------------
Reads stimulus files in from disk.  Both an input (f) and an
output (g) file are read, because some variants of these programs
are heteroassociative, and may have different f and g files.
-----------------------------------------------------------------------*/

#include <stdio.h>	/* feof fgets fopen fread fclose printf rewind */
#include <string.h>	/* strncpy */

/* Number_of_characters is related to Dimensionality of vectors */
#define Number_of_characters (int)(25)
#define Dimensionality       (int)(8*Number_of_characters)
/* Maximum_set_size is the size of each training set */
#define Maximum_set_size     (int)(100)
/* String_length refers to strings in the struct Stimulus.Name */
#define String_length  (int)(60)       /* Must be > Number_of_characters */

typedef float Vector[Dimensionality];
typedef char String[String_length+1];
typedef struct {
    String Name;
    Vector Val;
} Stimulus;

/* Defined in bsb.c */
/* Element [0] does not appear to be used anywhere */
Stimulus F_set[Maximum_set_size+1];
Stimulus G_set[Maximum_set_size+1];
Stimulus T_set[Maximum_set_size+1];
extern char FFileName[];
extern char GFileName[];
extern char TFileName[];

/* Defined in bsb_num.c */
void Blank_string(String S);
void Blank_f_set(void);
void Blank_g_set(void);
void Blank_t_set(void);
int Number_present_in_set(char Which_set);

/* Defined in bsb_readf.c */
static int Proceed;
static Stimulus Working_set[Maximum_set_size+1];
static char Which_file;	/* Can equal 'f','g' or 't' */
static void Blank_working_set(void);
static void Warnings(void);
static void Read_working_file(void);
static void Actually_read_the_file(void);
void Read_file(char Read_which_file);

/* Defined in bsb_vt100.c */
void Erase_line(void);
void Scroll_block(void);
void Bottom_line(void);
void Restore_scrolling(void);

/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Blank_working_set(void) {
    int i;	/* loop index */

    for(i=0; i<=Maximum_set_size; i++)
        Blank_string(Working_set[i].Name);
}   /* Blank_working_set */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Warnings(void) {
    char InputLine[80]; /* Keyboard response */
    char Input_char;	/* Keyboard response */

    if(Number_present_in_set(Which_file)>0) {
        do {
            Erase_line();
            printf("%c set contains records which will be erased. Proceed? ",
                Which_file);
            fgets(InputLine,sizeof(InputLine),stdin);
            Input_char=InputLine[0];
        } while(Input_char!='Y' && Input_char!='y' &&
                Input_char!='N' && Input_char!='n');
    } else
        Input_char='Y';
    Proceed=(Input_char=='Y' || Input_char=='y');
}   /* Warnings */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Read_working_file(void) {
    int i;		/* loop index */
    int Number_in_working_file;	/* number of records read from file */
    char File_name[80+1];	/* file to read */
    FILE *Working_file;

    Blank_working_set();

    if(Which_file=='F' || Which_file=='f') {
        Blank_f_set();
        strncpy(File_name,FFileName,80); File_name[80]='\0';
    } else if(Which_file=='G' || Which_file=='g') {
        Blank_g_set();
        strncpy(File_name,GFileName,80); File_name[80]='\0';
    } else if(Which_file=='T' || Which_file=='t') {
        Blank_t_set();
        strncpy(File_name,TFileName,80); File_name[80]='\0';
    }

    Bottom_line();
    Working_file=fopen(File_name,"r");
    if(Working_file==NULL) {
        printf("Error opening %c file \'%s\'.\n",Which_file,File_name);
    } else {
        for(i=0; i<Maximum_set_size && !feof(Working_file); ) {
            if(fread(&Working_set[i+1],sizeof(Stimulus),1,Working_file)==1)
                i++;	/* i indexes last successfully read stimulus */
        }
        fclose(Working_file);

        Number_in_working_file=i;
        printf("Number of stimuli in %c set was %d\n", 
                Which_file,Number_in_working_file);

        if(Which_file=='F' || Which_file=='f') {
            for(i=0; i<=Maximum_set_size; i++) F_set[i]=Working_set[i];
        } else if(Which_file=='G' || Which_file=='g') {
            for(i=0; i<=Maximum_set_size; i++) G_set[i]=Working_set[i];
        } else if(Which_file=='T' || Which_file=='t') {
            for(i=0; i<=Maximum_set_size; i++) T_set[i]=Working_set[i];
        }
    }
}   /* Read_working_file */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
static void Actually_read_the_file(void) {
    Bottom_line();
    Warnings();		/* May zap what's already there */

    if(Proceed)
        Read_working_file();
    else
        printf("No files read.\n");
}   /* Actually_read_the_file */


/*-----------------------------------------------------------------------
-----------------------------------------------------------------------*/
void Read_file(char Read_which_file) {
    int i;	/* loop index */

    Scroll_block();
    for(i=0; i<=Maximum_set_size; i++)
        Blank_string(Working_set[i].Name);

    if( Read_which_file=='F' || Read_which_file=='f' ||
        Read_which_file=='G' || Read_which_file=='g' ||
        Read_which_file=='T' || Read_which_file=='t') {
        Which_file=Read_which_file;
        Actually_read_the_file();
    } else if(Read_which_file=='A' || Read_which_file=='a') {
        Which_file='F'; Actually_read_the_file();
        Which_file='G'; Actually_read_the_file();
        Which_file='T'; Actually_read_the_file();
    } else {
        Bottom_line();
        printf("Can only read F, G or T files.\n");
        Proceed=0;
    }
    Restore_scrolling();
}   /* Read_file */

